code {
font-family: monospace;
}

[popover] {
border: 1px solid black;
box-shadow: 0 2px 5px rgba(0, 0, 0, 0.2);
opacity: 90%;
width: 100%;
}

[popover]:popover-open {
margin: 2rem auto auto auto;
}

::backdrop {
  background-color: rgba(0, 0, 0, 0.5);
}

a {
color: #000000;
font-family: "Bitter", serif;
font-weight: 600;
text-decoration: underline;
}

p {
font-family: "Lora", "Georgia", serif;
font-size: 1.4rem;
font-style: normal;
font-weight: 450;
}

dl div {
display: flex;
flex-direction: row;
place-items: center;
}

dt {
font-size: 1.4rem;
margin: 0.1rem auto 0.1rem 0.5rem;
}

dd {
font-size: 1.4rem;
margin: 0.1rem 0.5rem 0.1rem 0.8rem;
}

dt::after {
content: ":";
}

button[popovertarget] {
font-size: 2.4rem;
font-weight: bold;
margin: 0.6rem;
padding: 0.4rem;
}

.card p {
font-size: 1.1rem;
padding-left: 1.5rem;
text-wrap: balance;
}

.card span a {
font-weight: bold;
padding-top: 1.5rem;
text-decoration: underline;
}

em {
font-style: italic;
}

main p {
font-size: 1.25rem;
font-style: normal;
}

cite {
font-family: "Bitter", serif;
font-weight: 450;
font-style: normal;
}

cite ~ p {
font-weight: 350;
font-style: oblique;
}

